using System;
using Microsoft.SPOT;
using System.Threading;
using GHIElectronics.System;
using USBDbgLog;
using GHIElectronics.Hardware;

namespace StepperMotorMidiPlayer
{
    public class Program
    {
        public static USBDrive drive;
        public static PWM pwm;

        public static void Main()
        {
            Program.pwm = new PWM(PWM.PWMChannel.Channel_1);

            Debug.Print("Waiting for USB Drive...");
            SystemManager.Start(SystemEvent);

            while (drive == null)
            {
                Thread.Sleep(100);
            }

            while (!drive.IsVolumePresent)
            {
                Thread.Sleep(100);
            }

            Program.PlaySong();

            Thread.Sleep(Timeout.Infinite);
        }

        public static void SystemEvent(SystemEventType type, SystemEventArgs args)
        {
            if (type == SystemEventType.DevicesConnectionChanged)
            {
                if (args.isDeviceConnected == true)
                {
                    if (args.device.deviceType == GHIElectronics.System.SystemDevices.DeviceType.Drive)
                    {
                        drive = new USBDrive(args.device);
                    }
                }
            }
        }

        public static void PlaySong()
        {
            MIDIFileParser parser = new MIDIFileParser();
            parser.ParseFile(drive);

            for (int i = 0; i < parser.NoteEvents.Count; i++)
            {
                MIDIFileParser.NoteEvent noteEvent = (MIDIFileParser.NoteEvent)parser.NoteEvents[i];
                TimeSpan timeToWait = new TimeSpan(noteEvent.DeltaTime * parser.Tempo * TimeSpan.TicksPerMillisecond / 1000 / parser.TicksPerBeat);
                Debug.Print("Time To Wait: " + timeToWait.ToString());
                Thread.Sleep((int)(timeToWait.Ticks / TimeSpan.TicksPerMillisecond));

                if ((noteEvent.EventType & 0xF0) == 0x80)
                {
                    // Note Off
                    Debug.Print("NOTE OFF: " + noteEvent.NoteNumber.ToString());
                    Program.pwm.Set(0, 0);
                }
                else if ((noteEvent.EventType & 0xF0) == 0x90)
                {
                    // Note On
                    Debug.Print("NOTE ON: " + noteEvent.NoteNumber.ToString());
                    Program.PlayNote(noteEvent.NoteNumber);
                }
            }

            Program.pwm.Set(0, 0);
            Debug.Print("No more notes!");
        }

        public static void PlayNote(int noteNum)
        {
            int freq = Program.GetFreq(noteNum);
            Debug.Print("Freq: " + freq.ToString());
            Program.pwm.Set(freq, 50);
        }

        public static int GetFreq(int noteNum)
        {
            int diff = noteNum - 69;

            return (int)(440 * System.Math.Pow(2, (double)diff / 12));
        }
    }
}
