using System;
using Microsoft.SPOT;
using GHIElectronics.System.SystemDevices;
using Microsoft.SPOT.IO;
using System.IO;
using System.Text;
using StepperMotorMidiPlayer;

namespace USBDbgLog
{
    public class USBDrive : Drive
    {
        public bool IsVolumePresent {get; private set;}

        private VolumeInfo volume;

        public USBDrive(Device device)
            : base(device)
        {
            this.IsVolumePresent = false;
            RemovableMedia.Insert += new InsertEventHandler(RemovableMedia_Insert);
            RemovableMedia.Eject += new EjectEventHandler(RemovableMedia_Eject);
        }

        void RemovableMedia_Insert(object sender, MediaEventArgs e)
        {
            this.volume = e.Volume;
            Debug.Print("USB Thumb File System: " + e.Volume.FileSystem);

            if (!e.Volume.IsFormatted)
            {
                Debug.Print("Begin Format");
                e.Volume.Format("FAT32", 0);
                Debug.Print("End Format");
            }

            this.IsVolumePresent = true;
        }

        void RemovableMedia_Eject(object sender, MediaEventArgs e)
        {
            Debug.Print("USB Thumb Drive Removed");
            this.IsVolumePresent = false;
        }

        public byte[] ReadFile(ref int trackNum)
        {
            byte[] buffer;

            string[] files = Directory.GetFiles(this.volume.RootDirectory);

            if (files.Length > 0)
            {
                trackNum = Int32.Parse(files[0].Substring(files[0].Length - 1));
                Debug.Print("Track Number Requested: " + trackNum.ToString());
                FileStream stream = File.OpenRead(files[0]);
                buffer = new byte[stream.Length];
                stream.Read(buffer, 0, buffer.Length);
                stream.Close();

            }
            else
            {
                buffer = new byte[0];
            }

            return buffer;
        }
    }
}
